@@ -6,7 +6,7 @@
 //! [`Write`]: https://doc.rust-lang.org/stable/std/io/trait.Write.html
 
 use crate::{
-    BlockSize, CompressionMode, IntoInnerError, LargeWindowSize, ParameterError, Quality,
+    BlockSize, CompressionMode, IntoInnerError, LargeWindowSize, ParameterSetError, Quality,
     WindowSize,
 };
 use brotlic_sys::*;
@@ -77,7 +77,7 @@ impl BrotliEncoder {
         input: &[u8],
         output: &mut [u8],
         op: BrotliOperation,
-    ) -> Result<EncoderResult, EncoderError> {
+    ) -> Result<EncoderResult, EncodeError> {
         let mut input_ptr = input.as_ptr();
         let mut input_len = input.len();
         let mut output_ptr = output.as_mut_ptr();
@@ -101,12 +101,12 @@ impl BrotliEncoder {
 
             Ok(EncoderResult { bytes_read, bytes_written })
         } else {
-            Err(EncoderError)
+            Err(EncodeError)
         }
     }
 
     /// Convience function to call method [`Self::compress`] with only input and no output.
-    pub fn give_input(&mut self, input: &[u8], op: BrotliOperation) -> Result<usize, EncoderError> {
+    pub fn give_input(&mut self, input: &[u8], op: BrotliOperation) -> Result<usize, EncodeError> {
         Ok(self.compress(input, &mut [], op)?.bytes_read)
     }
 
@@ -117,7 +117,7 @@ impl BrotliEncoder {
     /// queried before flushing has been finalized. When flush is complete, output data will be
     /// sufficient for a decoder to reproduce all given input. Calling this function might resulting
     /// in a worse compression ratio, because the encoder is forced to emit all output immediately.
-    pub fn flush(&mut self) -> Result<(), EncoderError> {
+    pub fn flush(&mut self) -> Result<(), EncodeError> {
         self.give_op(BrotliOperation::Flush)
     }
 
@@ -128,7 +128,7 @@ impl BrotliEncoder {
     /// encoder is finished. Once this method has been called, no further input should be processed.
     ///
     /// For more information, see `BrotliEncoderOperation::BROTLI_OPERATION_FINISH`
-    pub fn finish(&mut self) -> Result<(), EncoderError> {
+    pub fn finish(&mut self) -> Result<(), EncodeError> {
         self.give_op(BrotliOperation::Finish)
     }
 
@@ -170,18 +170,33 @@ impl BrotliEncoder {
         &mut self,
         param: BrotliEncoderParameter,
         value: u32,
-    ) -> Result<(), ParameterError> {
+    ) -> Result<(), ParameterSetError> {
         let r = unsafe { BrotliEncoderSetParameter(self.state, param, value) };
 
-        if r != 0 { Ok(()) } else { Err(ParameterError::Generic) }
+        if r != 0 { Ok(()) } else { Err(ParameterSetError::Generic) }
     }
 
-    fn give_op(&mut self, op: BrotliOperation) -> Result<(), EncoderError> {
+    fn give_op(&mut self, op: BrotliOperation) -> Result<(), EncodeError> {
         self.give_input(&[], op)?;
         Ok(())
     }
 }
 
+impl Default for BrotliEncoder {
+    fn default() -> Self {
+        BrotliEncoder::new()
+    }
+}
+
+impl Drop for BrotliEncoder {
+    #[doc(alias = "BrotliEncoderDestroyInstance")]
+    fn drop(&mut self) {
+        unsafe {
+            BrotliEncoderDestroyInstance(self.state);
+        }
+    }
+}
+
 /// The operation for the encoder to process.
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum BrotliOperation {
@@ -199,21 +214,6 @@ pub enum BrotliOperation {
     Finish = BrotliEncoderOperation_BROTLI_OPERATION_FINISH as isize,
 }
 
-impl Default for BrotliEncoder {
-    fn default() -> Self {
-        BrotliEncoder::new()
-    }
-}
-
-impl Drop for BrotliEncoder {
-    #[doc(alias = "BrotliEncoderDestroyInstance")]
-    fn drop(&mut self) {
-        unsafe {
-            BrotliEncoderDestroyInstance(self.state);
-        }
-    }
-}
-
 /// Compression options to be used for a [`BrotliEncoder`].
 ///
 /// # Examples
@@ -224,8 +224,10 @@ impl Drop for BrotliEncoder {
 ///
 /// let encoder = BrotliEncoderOptions::new()
 ///     .mode(CompressionMode::Text)
-///     .quality(Quality::new(5).unwrap())
-///     .build();
+///     .quality(Quality::new(5)?)
+///     .build()?;
+///
+/// # Ok::<(), brotlic::ParameterSetError>(())
 /// ```
 pub struct BrotliEncoderOptions {
     mode: Option<CompressionMode>,
@@ -363,7 +365,7 @@ impl BrotliEncoderOptions {
     ///
     /// If any of the preconditions of the parameters are violated, an error is returned.
     #[doc(alias = "BrotliEncoderSetParameter")]
-    pub fn build(&self) -> Result<BrotliEncoder, ParameterError> {
+    pub fn build(&self) -> Result<BrotliEncoder, ParameterSetError> {
         let mut encoder = BrotliEncoder::new();
 
         if let Some(mode) = self.mode {
@@ -417,7 +419,7 @@ impl BrotliEncoderOptions {
 
         if let Some(postfix_bits) = self.postfix_bits {
             if postfix_bits > 3 {
-                return Err(ParameterError::InvalidPostfix);
+                return Err(ParameterSetError::InvalidPostfix);
             }
 
             let key = BrotliEncoderParameter_BROTLI_PARAM_NPOSTFIX;
@@ -431,7 +433,7 @@ impl BrotliEncoderOptions {
 
             if (direct_distance_codes > (15 << postfix)) ||
                 (direct_distance_codes & ((1 << postfix) - 1)) != 0 {
-                return Err(ParameterError::InvalidDirectDistanceCodes);
+                return Err(ParameterSetError::InvalidDirectDistanceCodes);
             }
 
             let key = BrotliEncoderParameter_BROTLI_PARAM_NDIRECT;
@@ -442,7 +444,7 @@ impl BrotliEncoderOptions {
 
         if let Some(stream_offset) = self.stream_offset {
             if stream_offset > (1 << 30) {
-                return Err(ParameterError::InvalidStreamOffset);
+                return Err(ParameterSetError::InvalidStreamOffset);
             }
 
             let key = BrotliEncoderParameter_BROTLI_PARAM_STREAM_OFFSET;
@@ -472,18 +474,18 @@ pub struct EncoderResult {
 
 /// An error returned by [`BrotliEncoder::compress`].
 #[derive(Debug)]
-pub struct EncoderError;
+pub struct EncodeError;
 
-impl error::Error for EncoderError {}
+impl error::Error for EncodeError {}
 
-impl fmt::Display for EncoderError {
+impl fmt::Display for EncodeError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("brotli encoder error")
     }
 }
 
-impl From<EncoderError> for io::Error {
-    fn from(err: EncoderError) -> Self {
+impl From<EncodeError> for io::Error {
+    fn from(err: EncodeError) -> Self {
         io::Error::new(io::ErrorKind::Other, err)
     }
 }
@@ -500,10 +502,10 @@ impl From<EncoderError> for io::Error {
 /// Suppose the file `test.txt` contains uncompressed text. Let's try to compress it:
 ///
 /// ```no_run
-/// # use std::fs::File;
-/// # use std::io::{self, Read};
-/// # use brotlic::DecompressorWriter;
-/// #
+/// use std::fs::File;
+/// use std::io::Read;
+/// use brotlic::DecompressorWriter;
+///
 /// let mut input = File::open("test.txt")?; // test.brotli is brotli compressed
 /// let mut output = Vec::new();
 ///
@@ -511,7 +513,7 @@ impl From<EncoderError> for io::Error {
 ///
 /// println!("Compressed length: {}", output.len());
 ///
-/// # Ok::<(), io::Error>(())
+/// # Ok::<(), std::io::Error>(())
 /// ```
 ///
 /// [`read`]: CompressorReader::read
@@ -537,6 +539,21 @@ impl<R: BufRead> CompressorReader<R> {
     }
 
     /// Creates a new `CompressorReader<R>` with a specified encoder.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::{BrotliEncoderOptions, CompressorReader, Quality, WindowSize};
+    ///
+    /// let encoder = BrotliEncoderOptions::new()
+    ///     .quality(Quality::new(6)?)
+    ///     .window_size(WindowSize::new(18)?)
+    ///     .build()?;
+    ///
+    /// let underlying_source = [1, 2, 3, 4, 5];
+    /// let writer = CompressorReader::with_encoder(encoder, underlying_source.as_slice());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
     pub fn with_encoder(encoder: BrotliEncoder, inner: R) -> Self {
         CompressorReader {
             inner,
@@ -623,10 +640,10 @@ impl<R: BufRead> Read for CompressorReader<R> {
 /// Let's compress some text file named `text.txt` and write the output to `test.brotli`:
 ///
 /// ```no_run
-/// # use std::fs::File;
-/// # use std::io;
-/// # use brotlic::CompressorWriter;
-/// #
+/// use std::fs::File;
+/// use std::io;
+/// use brotlic::CompressorWriter;
+///
 /// let mut input = File::open("test.txt")?; // test.txt is uncompressed
 /// let mut output = File::create("test.brotli")?;
 /// let mut compressed_output = CompressorWriter::new(output);
@@ -663,6 +680,21 @@ impl<W: Write> CompressorWriter<W> {
     }
 
     /// Creates a new `CompressorWriter<W>` with a specified encoder.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::{BrotliEncoderOptions, CompressorWriter, Quality, WindowSize};
+    ///
+    /// let encoder = BrotliEncoderOptions::new()
+    ///     .quality(Quality::new(4)?)
+    ///     .window_size(WindowSize::new(16)?)
+    ///     .build()?;
+    ///
+    /// let underlying_storage = Vec::new();
+    /// let writer = CompressorWriter::with_encoder(encoder, underlying_storage);
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
     pub fn with_encoder(encoder: BrotliEncoder, inner: W) -> Self {
         CompressorWriter {
             inner,
@@ -793,6 +825,34 @@ impl fmt::Display for WriterPanicked {
 mod tests {
     use super::*;
 
+    #[test]
+    fn invalid_quality() {
+        let invalid = Quality::new(12);
+
+        assert_eq!(invalid.unwrap_err(), ParameterSetError::InvalidQuality);
+    }
+
+    #[test]
+    fn invalid_window_size() {
+        let invalid = WindowSize::new(25);
+
+        assert_eq!(invalid.unwrap_err(), ParameterSetError::InvalidWindowSize);
+    }
+
+    #[test]
+    fn invalid_large_window_size() {
+        let invalid = LargeWindowSize::new(31);
+
+        assert_eq!(invalid.unwrap_err(), ParameterSetError::InvalidWindowSize);
+    }
+
+    #[test]
+    fn invalid_block_size() {
+        let invalid = BlockSize::new(25);
+
+        assert_eq!(invalid.unwrap_err(), ParameterSetError::InvalidBlockSize);
+    }
+
     #[test]
     fn valid_stream_offset() {
         let res = BrotliEncoderOptions::new()
@@ -808,7 +868,7 @@ mod tests {
             .stream_offset((1 << 30) + 2)
             .build();
 
-        assert_eq!(res.unwrap_err(), ParameterError::InvalidStreamOffset);
+        assert_eq!(res.unwrap_err(), ParameterSetError::InvalidStreamOffset);
     }
 
     #[test]
@@ -826,7 +886,7 @@ mod tests {
             .postfix_bits(7)
             .build();
 
-        assert_eq!(res.unwrap_err(), ParameterError::InvalidPostfix);
+        assert_eq!(res.unwrap_err(), ParameterSetError::InvalidPostfix);
     }
 
     #[test]
@@ -846,6 +906,6 @@ mod tests {
             .direct_distance_codes(120)
             .build();
 
-        assert_eq!(res.unwrap_err(), ParameterError::InvalidDirectDistanceCodes);
+        assert_eq!(res.unwrap_err(), ParameterSetError::InvalidDirectDistanceCodes);
     }
 }
