@@ -1,6 +1,6 @@
 //! # Brotlic
 //!
-//! Brotlic (or BrotlyC) is a thin wrapper around [brotli](https://github.com/google/brotli). It
+//! Brotlic (or BrotliC) is a thin wrapper around [brotli](https://github.com/google/brotli). It
 //! provides Rust bindings to all compression and decompression APIs. On the fly compression and
 //! decompression is supported for both `BufRead` and `Write` via [`CompressorReader<R>`,
 //! `CompressorWriter<W>`, `DecompressorReader<R>` and `DecompressorWriter<W>`. For low-level
@@ -95,7 +95,6 @@
 //!
 //! ```
 //! use brotlic::{BlockSize, BrotliEncoderOptions, CompressorWriter, Quality, WindowSize};
-//! # use brotlic::ParameterError;
 //!
 //! let encoder = BrotliEncoderOptions::new()
 //!     .quality(Quality::best())
@@ -105,7 +104,7 @@
 //!
 //! let compressed_writer = CompressorWriter::with_encoder(encoder, Vec::new());
 //!
-//! # Ok::<(), ParameterError>(())
+//! # Ok::<(), brotlic::ParameterSetError>(())
 //! ```
 //!
 //! It is recommended to not use the encoder directly but instead pass it onto the higher level
@@ -129,6 +128,7 @@ pub use decode::DecompressorWriter;
 use brotlic_sys::*;
 use std::os::raw::c_int;
 use std::{error, fmt, io};
+use std::io::ErrorKind;
 
 /// Quality level of the brotli compression
 ///
@@ -146,46 +146,121 @@ impl Quality {
     ///
     /// # Errors
     ///
-    /// An [`Err`] will be returned if the `value` is out of the range of valid qualities.
+    /// An [`Err`] will be returned if the `level` is out of the range of valid qualities.
     ///
     /// # Examples
     ///
     /// ```
     /// use brotlic::Quality;
     ///
-    /// let worst_quality = Quality::new(0).unwrap();
-    /// let best_quality = Quality::new(11).unwrap();
+    /// let worst_quality = Quality::new(0)?;
+    /// let best_quality = Quality::new(11)?;
     ///
     /// assert_eq!(worst_quality, Quality::worst());
     /// assert_eq!(best_quality, Quality::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn new(value: u8) -> Result<Quality, QualityError> {
-        match value {
-            BROTLI_MIN_QUALITY..=BROTLI_MAX_QUALITY => Ok(Quality(value)),
-            _ => Err(QualityError),
+    pub const fn new(level: u8) -> Result<Quality, ParameterSetError> {
+        match level {
+            BROTLI_MIN_QUALITY..=BROTLI_MAX_QUALITY => Ok(Quality(level)),
+            _ => Err(ParameterSetError::InvalidQuality),
         }
     }
 
+    /// Creates a new brotli compression quality without checking whether the integer represents a
+    /// valid quality. The range of valid qualities is from 0 to 11 inclusive, where 0 is the worst
+    /// possible quality and 11 is the best possible quality. Using any `level` outside of this
+    /// range will result in undefined behaviour.
+    ///
+    /// # Safety
+    ///
+    /// The `level` must be between 0 and 11.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::Quality;
+    ///
+    /// // SAFETY: 5 is within the range of valid qualities from 0 to 11
+    /// let quality = unsafe { Quality::new_unchecked(5) };
+    ///
+    /// assert_eq!(quality.level(), 5);
+    /// ```
+    pub const unsafe fn new_unchecked(level: u8) -> Quality {
+        Quality(level)
+    }
+
     /// The highest quality for brotli compression.
     ///
-    /// This quality yields maximum compression ratio at the expense of run-time speed.
-    pub fn best() -> Quality {
+    /// This quality yields maximum compression ratio at the expense of run-time speed. It's
+    /// currently set to 11.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::Quality;
+    ///
+    /// let best_quality = Quality::new(11)?;
+    ///
+    /// assert_eq!(best_quality, Quality::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn best() -> Quality {
         Quality(BROTLI_MAX_QUALITY)
     }
 
     /// The default quality to use for brotli compression.
     ///
-    /// This is current set to the best possible quality.
-    pub fn default() -> Quality {
+    /// This is set to the best possible quality 11.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::Quality;
+    ///
+    /// let default_quality = Quality::new(11)?;
+    ///
+    /// assert_eq!(default_quality, Quality::default());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn default() -> Quality {
         Quality(BROTLI_DEFAULT_QUALITY)
     }
 
     /// The worst quality to use for brotli compression.
     ///
     /// This quality yields the worst compression ratio while offering the highest run-time speed.
-    pub fn worst() -> Quality {
+    /// It's currently set to 0.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::Quality;
+    ///
+    /// let worst_quality = Quality::new(0)?;
+    ///
+    /// assert_eq!(worst_quality, Quality::worst());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn worst() -> Quality {
         Quality(BROTLI_MIN_QUALITY)
     }
+
+    /// Returns an integer representing the quality level.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::Quality;
+    ///
+    /// let quality = Quality::new(4)?;
+    ///
+    /// assert_eq!(quality.level(), 4);
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn level(&self) -> u8 {
+        self.0
+    }
 }
 
 impl Default for Quality {
@@ -198,22 +273,6 @@ impl Default for Quality {
     }
 }
 
-/// An error returned by [`Quality::new`].
-#[derive(Debug)]
-pub struct QualityError;
-
-impl fmt::Display for QualityError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "quality out of range (must be between {} and {} inclusive)",
-            BROTLI_MIN_QUALITY, BROTLI_MAX_QUALITY
-        )
-    }
-}
-
-impl error::Error for QualityError {}
-
 /// The sliding window size (in bits) to use for compression.
 ///
 /// Its maximum size is currently limited to 16 MiB, as specified in RFC7932 (Brotli proper).
@@ -227,7 +286,7 @@ impl error::Error for QualityError {}
 pub struct WindowSize(u8);
 
 impl WindowSize {
-    /// Consturcts a new sliding window size to use for brotli compression.
+    /// Constructs a new sliding window size to use for brotli compression.
     ///
     /// Valid `bits` range from 10 (1 KiB) to 24 (16 MiB) inclusive.
     ///
@@ -240,19 +299,43 @@ impl WindowSize {
     /// ```
     /// use brotlic::WindowSize;
     ///
-    /// let worst_size = WindowSize::new(10).unwrap();
-    /// let best_size = WindowSize::new(24).unwrap();
+    /// let worst_size = WindowSize::new(10)?;
+    /// let best_size = WindowSize::new(24)?;
     ///
     /// assert_eq!(worst_size, WindowSize::worst());
     /// assert_eq!(best_size, WindowSize::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn new(bits: u8) -> Result<WindowSize, WindowSizeError> {
+    pub const fn new(bits: u8) -> Result<WindowSize, ParameterSetError> {
         match bits {
             BROTLI_MIN_WINDOW_BITS..=BROTLI_MAX_WINDOW_BITS => Ok(WindowSize(bits)),
-            _ => Err(WindowSizeError),
+            _ => Err(ParameterSetError::InvalidWindowSize),
         }
     }
 
+    /// Constructs a new sliding window size (in bits) to use for brotli compression.
+    ///
+    /// Valid `bits` range from 10 (1 KiB) to 24 (16 MiB) inclusive. Using a number of `bits`
+    /// outside of that range results in undefined behaviour.
+    ///
+    /// # Safety
+    ///
+    /// The number of `bits` must be between 10 and 24.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::WindowSize;
+    ///
+    /// // SAFETY: 23 is within the valid range of 10 to 24 in window sizes
+    /// let window_size = unsafe { WindowSize::new_unchecked(23) };
+    ///
+    /// assert_eq!(window_size.bits(), 23);
+    /// ```
+    pub const unsafe fn new_unchecked(bits: u8) -> WindowSize {
+        WindowSize(bits)
+    }
+
     /// Constructs the best sliding window size to use for brotli compression.
     ///
     /// This is currently limited to 24 bits (16 MiB) due to RFC7932 (Brotli proper). To use larger
@@ -268,11 +351,12 @@ impl WindowSize {
     /// ```
     /// use brotlic::WindowSize;
     ///
-    /// let best_size = WindowSize::new(24).unwrap();
+    /// let best_size = WindowSize::new(24)?;
     ///
     /// assert_eq!(best_size, WindowSize::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn best() -> WindowSize {
+    pub const fn best() -> WindowSize {
         WindowSize(BROTLI_MAX_WINDOW_BITS)
     }
 
@@ -285,11 +369,12 @@ impl WindowSize {
     /// ```
     /// use brotlic::WindowSize;
     ///
-    /// let default_size = WindowSize::new(22).unwrap();
+    /// let default_size = WindowSize::new(22)?;
     ///
     /// assert_eq!(default_size, WindowSize::default());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn default() -> WindowSize {
+    pub const fn default() -> WindowSize {
         WindowSize(BROTLI_DEFAULT_WINDOW)
     }
 
@@ -302,13 +387,30 @@ impl WindowSize {
     /// ```
     /// use brotlic::WindowSize;
     ///
-    /// let worst_size = WindowSize::new(10).unwrap();
+    /// let worst_size = WindowSize::new(10)?;
     ///
     /// assert_eq!(worst_size, WindowSize::worst());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn worst() -> WindowSize {
+    pub const fn worst() -> WindowSize {
         WindowSize(BROTLI_MIN_WINDOW_BITS)
     }
+
+    /// Returns an integer representing the window size in bits.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::WindowSize;
+    ///
+    /// let window_size = WindowSize::new(24)?;
+    ///
+    /// assert_eq!(window_size.bits(), 24);
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn bits(&self) -> u8 {
+        self.0
+    }
 }
 
 impl Default for WindowSize {
@@ -322,7 +424,7 @@ impl Default for WindowSize {
 }
 
 impl TryFrom<LargeWindowSize> for WindowSize {
-    type Error = WindowSizeError;
+    type Error = ParameterSetError;
 
     /// Attempts to construct a [`WindowSize`] from a [`LargeWindowSize`].
     ///
@@ -337,22 +439,6 @@ impl TryFrom<LargeWindowSize> for WindowSize {
     }
 }
 
-/// An error returned by [`WindowSize::new`].
-#[derive(Debug)]
-pub struct WindowSizeError;
-
-impl fmt::Display for WindowSizeError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "window size out of range (must be between {} and {} inclusive)",
-            BROTLI_MIN_WINDOW_BITS, BROTLI_MAX_WINDOW_BITS
-        )
-    }
-}
-
-impl error::Error for WindowSizeError {}
-
 /// The large sliding window size (in bits) to use for compression.
 ///
 /// Note that using a large sliding window size for a particular compressor requires explicit
@@ -365,7 +451,7 @@ impl error::Error for WindowSizeError {}
 pub struct LargeWindowSize(u8);
 
 impl LargeWindowSize {
-    /// Consturcts a new large sliding window size (in bits) to use for brotli compression.
+    /// Constructs a new large sliding window size (in bits) to use for brotli compression.
     ///
     /// Valid `bits` range from 10 (1 KiB) to 30 (1 GiB) inclusive.
     ///
@@ -378,19 +464,43 @@ impl LargeWindowSize {
     /// ```
     /// use brotlic::LargeWindowSize;
     ///
-    /// let worst_size = LargeWindowSize::new(10).unwrap();
-    /// let best_size = LargeWindowSize::new(30).unwrap();
+    /// let worst_size = LargeWindowSize::new(10)?;
+    /// let best_size = LargeWindowSize::new(30)?;
     ///
     /// assert_eq!(worst_size, LargeWindowSize::worst());
     /// assert_eq!(best_size, LargeWindowSize::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn new(bits: u8) -> Result<LargeWindowSize, LargeWindowSizeError> {
+    pub const fn new(bits: u8) -> Result<LargeWindowSize, ParameterSetError> {
         match bits {
             BROTLI_MIN_WINDOW_BITS..=BROTLI_LARGE_MAX_WINDOW_BITS => Ok(LargeWindowSize(bits)),
-            _ => Err(LargeWindowSizeError),
+            _ => Err(ParameterSetError::InvalidWindowSize),
         }
     }
 
+    /// Constructs a new large sliding window size (in bits) to use for brotli compression.
+    ///
+    /// Valid `bits` range from 10 (1 KiB) to 30 (1 GiB) inclusive. Using a number of `bits` outside
+    /// of that range results in undefined behaviour.
+    ///
+    /// # Safety
+    ///
+    /// The number of `bits` must be between 10 and 30.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::LargeWindowSize;
+    ///
+    /// // SAFETY: 28 is within the valid range of 10 to 30 in large window sizes
+    /// let window_size = unsafe { LargeWindowSize::new_unchecked(28) };
+    ///
+    /// assert_eq!(window_size.bits(), 28);
+    /// ```
+    pub const unsafe fn new_unchecked(bits: u8) -> LargeWindowSize {
+        LargeWindowSize(bits)
+    }
+
     /// Constructs the best large sliding window size to use for brotli compression.
     ///
     /// This is currently set to 30 bits (1 GiB). Note that this requires explicit support by the
@@ -401,11 +511,12 @@ impl LargeWindowSize {
     /// ```
     /// use brotlic::LargeWindowSize;
     ///
-    /// let best_size = LargeWindowSize::new(30).unwrap();
+    /// let best_size = LargeWindowSize::new(30)?;
     ///
     /// assert_eq!(best_size, LargeWindowSize::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn best() -> LargeWindowSize {
+    pub const fn best() -> LargeWindowSize {
         LargeWindowSize(BROTLI_LARGE_MAX_WINDOW_BITS)
     }
 
@@ -418,11 +529,12 @@ impl LargeWindowSize {
     /// ```
     /// use brotlic::LargeWindowSize;
     ///
-    /// let default_size = LargeWindowSize::new(22).unwrap();
+    /// let default_size = LargeWindowSize::new(22)?;
     ///
     /// assert_eq!(default_size, LargeWindowSize::default());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn default() -> LargeWindowSize {
+    pub const fn default() -> LargeWindowSize {
         LargeWindowSize(BROTLI_DEFAULT_WINDOW)
     }
 
@@ -435,13 +547,30 @@ impl LargeWindowSize {
     /// ```
     /// use brotlic::LargeWindowSize;
     ///
-    /// let worst_size = LargeWindowSize::new(10).unwrap();
+    /// let worst_size = LargeWindowSize::new(10)?;
     ///
     /// assert_eq!(worst_size, LargeWindowSize::worst());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn worst() -> LargeWindowSize {
+    pub const fn worst() -> LargeWindowSize {
         LargeWindowSize(BROTLI_MIN_WINDOW_BITS)
     }
+
+    /// Returns an integer representing the large window size in bits.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::LargeWindowSize;
+    ///
+    /// let window_size = LargeWindowSize::new(28)?;
+    ///
+    /// assert_eq!(window_size.bits(), 28);
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn bits(&self) -> u8 {
+        self.0
+    }
 }
 
 impl Default for LargeWindowSize {
@@ -464,22 +593,6 @@ impl From<WindowSize> for LargeWindowSize {
     }
 }
 
-/// An error returned by [`LargeWindowSize::new`].
-#[derive(Debug)]
-pub struct LargeWindowSizeError;
-
-impl fmt::Display for LargeWindowSizeError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "large window size out of range (must be between {} and {} inclusive)",
-            BROTLI_MIN_WINDOW_BITS, BROTLI_LARGE_MAX_WINDOW_BITS
-        )
-    }
-}
-
-impl error::Error for LargeWindowSizeError {}
-
 /// The recommended input block size (in bits) to use for compression.
 ///
 /// The compressor may reduce this value at its leisure, for example when the input size is small.
@@ -502,19 +615,42 @@ impl BlockSize {
     /// ```
     /// use brotlic::BlockSize;
     ///
-    /// let worst_size = BlockSize::new(16).unwrap();
-    /// let best_size = BlockSize::new(24).unwrap();
+    /// let worst_size = BlockSize::new(16)?;
+    /// let best_size = BlockSize::new(24)?;
     ///
     /// assert_eq!(worst_size, BlockSize::worst());
     /// assert_eq!(best_size, BlockSize::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
-    pub fn new(bits: u8) -> Result<BlockSize, BlockSizeError> {
+    pub const fn new(bits: u8) -> Result<BlockSize, ParameterSetError> {
         match bits {
             BROTLI_MIN_INPUT_BLOCK_BITS..=BROTLI_MAX_INPUT_BLOCK_BITS => Ok(BlockSize(bits)),
-            _ => Err(BlockSizeError),
+            _ => Err(ParameterSetError::InvalidBlockSize),
         }
     }
 
+    /// Constructs a new block size (in bits) to use for brotli compression.
+    ///
+    /// Valid `bits` range from 16 to 24 inclusive. Using any number of bits outside of that range
+    /// results in undefined behaviour.
+    ///
+    /// # Safety
+    ///
+    /// The number of `bits` must be between 16 and 24.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::BlockSize;
+    ///
+    /// let block_size = unsafe{ BlockSize::new_unchecked(22) };
+    ///
+    /// assert_eq!(block_size.bits(), 22);
+    /// ```
+    pub const fn new_unchecked(bits: u8) -> BlockSize {
+        BlockSize(bits)
+    }
+
     /// Constructs the best block size (in bits) to use for brotli compression.
     ///
     /// This will allow better compression at the expense of memory usage. Currently it is set to
@@ -525,9 +661,10 @@ impl BlockSize {
     /// ```
     /// use brotlic::BlockSize;
     ///
-    /// let best_size = BlockSize::new(24).unwrap();
+    /// let best_size = BlockSize::new(24)?;
     ///
     /// assert_eq!(best_size, BlockSize::best());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
     pub fn best() -> BlockSize {
         BlockSize(BROTLI_MAX_INPUT_BLOCK_BITS)
@@ -543,31 +680,32 @@ impl BlockSize {
     /// ```
     /// use brotlic::BlockSize;
     ///
-    /// let worst_size = BlockSize::new(16).unwrap();
+    /// let worst_size = BlockSize::new(16)?;
     ///
     /// assert_eq!(worst_size, BlockSize::worst());
+    /// # Ok::<(), brotlic::ParameterSetError>(())
     /// ```
     pub fn worst() -> BlockSize {
         BlockSize(BROTLI_MIN_INPUT_BLOCK_BITS)
     }
-}
 
-/// An error returned by [`BlockSize::new`].
-#[derive(Debug)]
-pub struct BlockSizeError;
-
-impl fmt::Display for BlockSizeError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "block size out of range (must be between {} and {} inclusive)",
-            BROTLI_MIN_INPUT_BLOCK_BITS, BROTLI_MAX_INPUT_BLOCK_BITS
-        )
+    /// Returns an integer representing the block size in bits.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use brotlic::BlockSize;
+    ///
+    /// let block_size = BlockSize::new(23)?;
+    ///
+    /// assert_eq!(block_size.bits(), 23);
+    /// # Ok::<(), brotlic::ParameterSetError>(())
+    /// ```
+    pub const fn bits(&self) -> u8 {
+        self.0
     }
 }
 
-impl error::Error for BlockSizeError {}
-
 /// Allows to tune a brotli compressor for a specific type of input.
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum CompressionMode {
@@ -593,40 +731,53 @@ impl Default for CompressionMode {
 
 /// An error returned by [`compress`].
 #[derive(Debug)]
-pub struct CompressionError;
+pub struct CompressError;
 
-impl fmt::Display for CompressionError {
+impl fmt::Display for CompressError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("buffer was too small or compression error occurred")
     }
 }
 
-impl error::Error for CompressionError {}
+impl error::Error for CompressError {}
+
+impl From<CompressError> for io::Error {
+    fn from(err: CompressError) -> Self {
+        io::Error::new(ErrorKind::Other, err)
+    }
+}
 
 /// a specialized [`Result`] type returned by [`compress`].
-pub type CompressionResult<T> = Result<T, CompressionError>;
+pub type CompressionResult<T> = Result<T, CompressError>;
 
 /// An error returned by [`decompress`].
 #[derive(Debug)]
-pub struct DecompressionError;
+pub struct DecompressError;
 
-impl fmt::Display for DecompressionError {
+impl fmt::Display for DecompressError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("buffer was too small or decompression error occurred")
     }
 }
 
-impl error::Error for DecompressionError {}
+impl error::Error for DecompressError {}
+
+impl From<DecompressError> for io::Error {
+    fn from(err: DecompressError) -> Self {
+        io::Error::new(ErrorKind::Other, err)
+    }
+}
 
 /// a specialized [`Result`] type returned by [`decompress`].
-pub type DecompressionResult<T> = Result<T, DecompressionError>;
+pub type DecompressionResult<T> = Result<T, DecompressError>;
 
 /// An error returned by [`BrotliEncoderOptions::build`] and [`BrotliDecoderOptions::build`]
 ///
 /// [`BrotliEncoderOptions::build`]: encode::BrotliEncoderOptions::build
 /// [`BrotliDecoderOptions::build`]: decode::BrotliDecoderOptions::build
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
-pub enum ParameterError {
+#[non_exhaustive]
+pub enum ParameterSetError {
     /// The encoder or decoder returned an error.
     ///
     /// This error originates from `BrotliEncoderSetParameter` or `BrotliDecoderSetParameter` being
@@ -641,24 +792,39 @@ pub enum ParameterError {
 
     /// The stream offset was beyond its maximum offset.
     InvalidStreamOffset,
+
+    /// The quality was out of range.
+    InvalidQuality,
+
+    /// sliding window size bits were out of range.
+    InvalidWindowSize,
+
+    /// Block size bits were out of range.
+    InvalidBlockSize,
 }
 
-impl fmt::Display for ParameterError {
+impl fmt::Display for ParameterSetError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            ParameterError::Generic =>
+            ParameterSetError::Generic =>
                 f.write_str("invalid parameter"),
-            ParameterError::InvalidPostfix =>
+            ParameterSetError::InvalidPostfix =>
                 f.write_str("invalid number of postfix bits"),
-            ParameterError::InvalidDirectDistanceCodes =>
+            ParameterSetError::InvalidDirectDistanceCodes =>
                 f.write_str("invalid number of direct distance codes"),
-            ParameterError::InvalidStreamOffset =>
+            ParameterSetError::InvalidStreamOffset =>
                 f.write_str("stream offset was out of range"),
+            ParameterSetError::InvalidQuality =>
+                f.write_str("quality out of range"),
+            ParameterSetError::InvalidWindowSize =>
+                f.write_str("window size out of range"),
+            ParameterSetError::InvalidBlockSize =>
+                f.write_str("block size out of range"),
         }
     }
 }
 
-impl error::Error for ParameterError {}
+impl error::Error for ParameterSetError {}
 
 /// Read all bytes from `input` and compress them into `output`, returning how many bytes were
 /// written.
@@ -691,9 +857,10 @@ impl error::Error for ParameterError {}
 ///      Quality::default(),
 ///      WindowSize::default(),
 ///      CompressionMode::Generic
-/// ).unwrap();
+/// )?;
 ///
 /// assert!(bytes_written < input.len());
+/// # Ok::<(), brotlic::CompressError>(())
 /// ```
 #[doc(alias = "BrotliEncoderCompress")]
 pub fn compress(
@@ -720,7 +887,7 @@ pub fn compress(
     if res != 0 {
         Ok(output_size)
     } else {
-        Err(CompressionError)
+        Err(CompressError)
     }
 }
 
@@ -768,13 +935,14 @@ pub fn compress_bound(input_size: usize, quality: Quality) -> Option<usize> {
 ///      Quality::default(),
 ///      WindowSize::default(),
 ///      CompressionMode::Generic
-/// ).unwrap();
+/// )?;
 ///
 /// let encoded = &encoded[..bytes_written];
-/// let bytes_written = decompress(encoded, decoded.as_mut_slice()).unwrap();
+/// let bytes_written = decompress(encoded, decoded.as_mut_slice())?;
 /// let decoded = &decoded[..bytes_written];
 ///
 /// assert_eq!(input, decoded);
+/// # Ok::<(), std::io::Error>(())
 /// ```
 #[doc(alias = "BrotliDecoderDecompress")]
 pub fn decompress(input: &[u8], output: &mut [u8]) -> DecompressionResult<usize> {
@@ -792,7 +960,7 @@ pub fn decompress(input: &[u8], output: &mut [u8]) -> DecompressionResult<usize>
     if res == BrotliDecoderResult_BROTLI_DECODER_RESULT_SUCCESS {
         Ok(output_size)
     } else {
-        Err(DecompressionError)
+        Err(DecompressError)
     }
 }
 
